@@ -13,44 +13,44 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""A utility class to send to and recv from a non-blocking socket,
-using tornado.
+"""A utility class for event-based messaging on a zmq socket using tornado.
 
 .. seealso::
 
     - :mod:`zmq.asyncio`
     - :mod:`zmq.eventloop.future`
-
 """
 
-from __future__ import with_statement
+import asyncio
 import pickle
-import sys
 import warnings
 from queue import Queue
+from typing import (
+    Any,
+    Awaitable,
+    Callable,
+    List,
+    Optional,
+    Sequence,
+    Union,
+    cast,
+    overload,
+)
+
+from tornado.ioloop import IOLoop
+from tornado.log import gen_log
 
 import zmq
+import zmq._future
+from zmq import POLLIN, POLLOUT
+from zmq._typing import Literal
 from zmq.utils import jsonapi
 
 
-from .ioloop import IOLoop, gen_log
-
-try:
-    from tornado.stack_context import wrap as stack_context_wrap  # type: ignore
-except ImportError:
-    if "zmq.eventloop.minitornado" in sys.modules:
-        from .minitornado.stack_context import wrap as stack_context_wrap  # type: ignore
-    else:
-        # tornado 5 deprecates stack_context,
-        # tornado 6 removes it
-        def stack_context_wrap(callback):
-            return callback
-
-
-class ZMQStream(object):
+class ZMQStream:
     """A utility class to register callbacks when a zmq socket sends and receives
 
-    For use with zmq.eventloop.ioloop
+    For use with tornado IOLoop.
 
     There are three main methods
 
@@ -60,7 +60,7 @@ class ZMQStream(object):
         register a callback to be run every time the socket has something to receive
     * **on_send(callback):**
         register a callback to be run every time you call send
-    * **send(self, msg, flags=0, copy=False, callback=None):**
+    * **send_multipart(self, msg, flags=0, copy=False, callback=None):**
         perform a send that will trigger the callback
         if callback is passed, on_send is also called.
 
@@ -82,25 +82,60 @@ class ZMQStream(object):
     >>> stream.bind is stream.socket.bind
     True
 
+
+    .. versionadded:: 25
+
+        send/recv callbacks can be coroutines.
+
+    .. versionchanged:: 25
+
+        ZMQStreams only support base zmq.Socket classes (this has always been true, but not enforced).
+        If ZMQStreams are created with e.g. async Socket subclasses,
+        a RuntimeWarning will be shown,
+        and the socket cast back to the default zmq.Socket
+        before connecting events.
+
+        Previously, using async sockets (or any zmq.Socket subclass) would result in undefined behavior for the
+        arguments passed to callback functions.
+        Now, the callback functions reliably get the return value of the base `zmq.Socket` send/recv_multipart methods
+        (the list of message frames).
     """
 
-    socket = None
-    io_loop = None
-    poller = None
-    _send_queue = None
-    _recv_callback = None
-    _send_callback = None
-    _close_callback = None
-    _state = 0
-    _flushed = False
-    _recv_copy = False
-    _fd = None
+    socket: zmq.Socket
+    io_loop: IOLoop
+    poller: zmq.Poller
+    _send_queue: Queue
+    _recv_callback: Optional[Callable]
+    _send_callback: Optional[Callable]
+    _close_callback: Optional[Callable]
+    _state: int = 0
+    _flushed: bool = False
+    _recv_copy: bool = False
+    _fd: int
 
-    def __init__(self, socket, io_loop=None):
+    def __init__(self, socket: "zmq.Socket", io_loop: Optional[IOLoop] = None):
+        if isinstance(socket, zmq._future._AsyncSocket):
+            warnings.warn(
+                f"""ZMQStream only supports the base zmq.Socket class.
+
+                Use zmq.Socket(shadow=other_socket)
+                or `ctx.socket(zmq.{socket._type_name}, socket_class=zmq.Socket)`
+                to create a base zmq.Socket object,
+                no matter what other kind of socket your Context creates.
+                """,
+                RuntimeWarning,
+                stacklevel=2,
+            )
+            # shadow back to base zmq.Socket,
+            # otherwise callbacks like `on_recv` will get the wrong types.
+            socket = zmq.Socket(shadow=socket)
         self.socket = socket
+
+        # IOLoop.current() is deprecated if called outside the event loop
+        # that means
         self.io_loop = io_loop or IOLoop.current()
         self.poller = zmq.Poller()
-        self._fd = self.socket.FD
+        self._fd = cast(int, self.socket.FD)
 
         self._send_queue = Queue()
         self._recv_callback = None
@@ -135,11 +170,52 @@ class ZMQStream(object):
         """DEPRECATED, does nothing"""
         gen_log.warn("on_err does nothing, and will be removed")
 
-    def on_err(self, callback):
+    def on_err(self, callback: Callable):
         """DEPRECATED, does nothing"""
         gen_log.warn("on_err does nothing, and will be removed")
 
-    def on_recv(self, callback, copy=True):
+    @overload
+    def on_recv(
+        self,
+        callback: Callable[[List[bytes]], Any],
+    ) -> None:
+        ...
+
+    @overload
+    def on_recv(
+        self,
+        callback: Callable[[List[bytes]], Any],
+        copy: Literal[True],
+    ) -> None:
+        ...
+
+    @overload
+    def on_recv(
+        self,
+        callback: Callable[[List[zmq.Frame]], Any],
+        copy: Literal[False],
+    ) -> None:
+        ...
+
+    @overload
+    def on_recv(
+        self,
+        callback: Union[
+            Callable[[List[zmq.Frame]], Any],
+            Callable[[List[bytes]], Any],
+        ],
+        copy: bool = ...,
+    ):
+        ...
+
+    def on_recv(
+        self,
+        callback: Union[
+            Callable[[List[zmq.Frame]], Any],
+            Callable[[List[bytes]], Any],
+        ],
+        copy: bool = True,
+    ) -> None:
         """Register a callback for when a message is ready to recv.
 
         There can be only one callback registered at a time, so each
@@ -167,14 +243,55 @@ class ZMQStream(object):
 
         self._check_closed()
         assert callback is None or callable(callback)
-        self._recv_callback = stack_context_wrap(callback)
+        self._recv_callback = callback
         self._recv_copy = copy
         if callback is None:
             self._drop_io_state(zmq.POLLIN)
         else:
             self._add_io_state(zmq.POLLIN)
 
-    def on_recv_stream(self, callback, copy=True):
+    @overload
+    def on_recv_stream(
+        self,
+        callback: Callable[["ZMQStream", List[bytes]], Any],
+    ) -> None:
+        ...
+
+    @overload
+    def on_recv_stream(
+        self,
+        callback: Callable[["ZMQStream", List[bytes]], Any],
+        copy: Literal[True],
+    ) -> None:
+        ...
+
+    @overload
+    def on_recv_stream(
+        self,
+        callback: Callable[["ZMQStream", List[zmq.Frame]], Any],
+        copy: Literal[False],
+    ) -> None:
+        ...
+
+    @overload
+    def on_recv_stream(
+        self,
+        callback: Union[
+            Callable[["ZMQStream", List[zmq.Frame]], Any],
+            Callable[["ZMQStream", List[bytes]], Any],
+        ],
+        copy: bool = ...,
+    ):
+        ...
+
+    def on_recv_stream(
+        self,
+        callback: Union[
+            Callable[["ZMQStream", List[zmq.Frame]], Any],
+            Callable[["ZMQStream", List[bytes]], Any],
+        ],
+        copy: bool = True,
+    ):
         """Same as on_recv, but callback will get this stream as first argument
 
         callback must take exactly two arguments, as it will be called as::
@@ -186,9 +303,15 @@ class ZMQStream(object):
         if callback is None:
             self.stop_on_recv()
         else:
-            self.on_recv(lambda msg: callback(self, msg), copy=copy)
 
-    def on_send(self, callback):
+            def stream_callback(msg):
+                return callback(self, msg)
+
+            self.on_recv(stream_callback, copy=copy)
+
+    def on_send(
+        self, callback: Callable[[Sequence[Any], Optional[zmq.MessageTracker]], Any]
+    ):
         """Register a callback to be called on each send
 
         There will be two arguments::
@@ -226,9 +349,14 @@ class ZMQStream(object):
 
         self._check_closed()
         assert callback is None or callable(callback)
-        self._send_callback = stack_context_wrap(callback)
+        self._send_callback = callback
 
-    def on_send_stream(self, callback):
+    def on_send_stream(
+        self,
+        callback: Callable[
+            ["ZMQStream", Sequence[Any], Optional[zmq.MessageTracker]], Any
+        ],
+    ):
         """Same as on_send, but callback will get this stream as first argument
 
         Callback will be passed three arguments::
@@ -251,8 +379,14 @@ class ZMQStream(object):
         )
 
     def send_multipart(
-        self, msg, flags=0, copy=True, track=False, callback=None, **kwargs
-    ):
+        self,
+        msg: Sequence[Any],
+        flags: int = 0,
+        copy: bool = True,
+        track: bool = False,
+        callback: Optional[Callable] = None,
+        **kwargs: Any,
+    ) -> None:
         """Send a multipart message, optionally also register a new callback for sends.
         See zmq.socket.send_multipart for details.
         """
@@ -266,7 +400,14 @@ class ZMQStream(object):
             self.on_send(lambda *args: None)
         self._add_io_state(zmq.POLLOUT)
 
-    def send_string(self, u, flags=0, encoding='utf-8', callback=None, **kwargs):
+    def send_string(
+        self,
+        u: str,
+        flags: int = 0,
+        encoding: str = 'utf-8',
+        callback: Optional[Callable] = None,
+        **kwargs: Any,
+    ):
         """Send a unicode message with an encoding.
         See zmq.socket.send_unicode for details.
         """
@@ -276,14 +417,27 @@ class ZMQStream(object):
 
     send_unicode = send_string
 
-    def send_json(self, obj, flags=0, callback=None, **kwargs):
+    def send_json(
+        self,
+        obj: Any,
+        flags: int = 0,
+        callback: Optional[Callable] = None,
+        **kwargs: Any,
+    ):
         """Send json-serialized version of an object.
         See zmq.socket.send_json for details.
         """
         msg = jsonapi.dumps(obj)
         return self.send(msg, flags=flags, callback=callback, **kwargs)
 
-    def send_pyobj(self, obj, flags=0, protocol=-1, callback=None, **kwargs):
+    def send_pyobj(
+        self,
+        obj: Any,
+        flags: int = 0,
+        protocol: int = -1,
+        callback: Optional[Callable] = None,
+        **kwargs: Any,
+    ):
         """Send a Python object as a message using pickle to serialize.
 
         See zmq.socket.send_json for details.
@@ -295,7 +449,7 @@ class ZMQStream(object):
         """callback for unsetting _flushed flag."""
         self._flushed = False
 
-    def flush(self, flag=zmq.POLLIN | zmq.POLLOUT, limit=None):
+    def flush(self, flag: int = zmq.POLLIN | zmq.POLLOUT, limit: Optional[int] = None):
         """Flush pending messages.
 
         This method safely handles all pending incoming and/or outgoing messages,
@@ -347,13 +501,13 @@ class ZMQStream(object):
         events = self.poller.poll(0)
         while events and (not limit or count < limit):
             s, event = events[0]
-            if event & zmq.POLLIN:  # receiving
+            if event & POLLIN:  # receiving
                 self._handle_recv()
                 count += 1
                 if self.socket is None:
                     # break if socket was closed during callback
                     break
-            if event & zmq.POLLOUT and self.sending():
+            if event & POLLOUT and self.sending():
                 self._handle_send()
                 count += 1
                 if self.socket is None:
@@ -379,11 +533,11 @@ class ZMQStream(object):
         self._rebuild_io_state()
         return count
 
-    def set_close_callback(self, callback):
+    def set_close_callback(self, callback: Optional[Callable]):
         """Call the given callback when the stream is closed."""
-        self._close_callback = stack_context_wrap(callback)
+        self._close_callback = callback
 
-    def close(self, linger=None):
+    def close(self, linger: Optional[int] = None) -> None:
         """Close this stream."""
         if self.socket is not None:
             if self.socket.closed:
@@ -401,19 +555,19 @@ class ZMQStream(object):
             else:
                 self.io_loop.remove_handler(self.socket)
                 self.socket.close(linger)
-            self.socket = None
+            self.socket = None  # type: ignore
             if self._close_callback:
                 self._run_callback(self._close_callback)
 
-    def receiving(self):
+    def receiving(self) -> bool:
         """Returns True if we are currently receiving from the stream."""
         return self._recv_callback is not None
 
-    def sending(self):
+    def sending(self) -> bool:
         """Returns True if we are currently sending to the stream."""
         return not self._send_queue.empty()
 
-    def closed(self):
+    def closed(self) -> bool:
         if self.socket is None:
             return True
         if self.socket.closed:
@@ -421,20 +575,35 @@ class ZMQStream(object):
             # trigger our cleanup
             self.close()
             return True
+        return False
 
     def _run_callback(self, callback, *args, **kwargs):
         """Wrap running callbacks in try/except to allow us to
         close our socket."""
         try:
-            # Use a NullContext to ensure that all StackContexts are run
-            # inside our blanket exception handler rather than outside.
-            callback(*args, **kwargs)
+            f = callback(*args, **kwargs)
+            if isinstance(f, Awaitable):
+                f = asyncio.ensure_future(f)
+            else:
+                f = None
         except Exception:
             gen_log.error("Uncaught exception in ZMQStream callback", exc_info=True)
             # Re-raise the exception so that IOLoop.handle_callback_exception
             # can see it and log the error
             raise
 
+        if f is not None:
+            # handle async callbacks
+            def _log_error(f):
+                try:
+                    f.result()
+                except Exception:
+                    gen_log.error(
+                        "Uncaught exception in ZMQStream callback", exc_info=True
+                    )
+
+            f.add_done_callback(_log_error)
+
     def _handle_events(self, fd, events):
         """This method is the actual handler for IOLoop, that gets called whenever
         an event on my socket is posted. It dispatches to _handle_recv, etc."""
@@ -445,6 +614,19 @@ class ZMQStream(object):
             zmq_events = self.socket.EVENTS
         except zmq.ContextTerminated:
             gen_log.warning("Got events for stream %s after terminating context", self)
+            # trigger close check, this will unregister callbacks
+            self.closed()
+            return
+        except zmq.ZMQError as e:
+            # run close check
+            # shadow sockets may have been closed elsewhere,
+            # which should show up as ENOTSOCK here
+            if self.closed():
+                gen_log.warning(
+                    "Got events for stream %s attached to closed socket: %s", self, e
+                )
+            else:
+                gen_log.error("Error getting events for %s: %s", self, e)
             return
         try:
             # dispatch events:
@@ -500,7 +682,7 @@ class ZMQStream(object):
 
     def _check_closed(self):
         if not self.socket:
-            raise IOError("Stream is closed")
+            raise OSError("Stream is closed")
 
     def _rebuild_io_state(self):
         """rebuild io state based on self.sending() and receiving()"""
