@@ -1,4 +1,3 @@
-# -*- coding: utf8 -*-
 # Copyright (C) PyZMQ Developers
 # Distributed under the terms of the Modified BSD License.
 
@@ -7,19 +6,17 @@ import errno
 import json
 import os
 import platform
-import time
-import warnings
 import socket
 import sys
-
+import time
+import warnings
 from unittest import mock
 
 import pytest
 from pytest import mark
 
 import zmq
-from zmq.tests import BaseZMQTestCase, SkipTest, have_gevent, GreenTest, skip_pypy
-from zmq.utils.strtypes import unicode
+from zmq.tests import BaseZMQTestCase, GreenTest, SkipTest, have_gevent, skip_pypy
 
 pypy = platform.python_implementation().lower() == 'pypy'
 windows = platform.platform().lower().startswith('windows')
@@ -38,7 +35,7 @@ class TestSocket(BaseZMQTestCase):
         self.assertRaisesErrno(zmq.EPROTONOSUPPORT, s.connect, 'ftl://a')
         self.assertRaisesErrno(zmq.EINVAL, s.bind, 'tcp://')
         s.close()
-        del ctx
+        ctx.term()
 
     def test_context_manager(self):
         url = 'inproc://a'
@@ -50,10 +47,10 @@ class TestSocket(BaseZMQTestCase):
                     msg = b'hi'
                     a.send(msg)
                     rcvd = self.recv(b)
-                    self.assertEqual(rcvd, msg)
-                self.assertEqual(b.closed, True)
-            self.assertEqual(a.closed, True)
-        self.assertEqual(ctx.closed, True)
+                    assert rcvd == msg
+                assert b.closed == True
+            assert a.closed == True
+        assert ctx.closed == True
 
     def test_connectbind_context_managers(self):
         url = 'inproc://a'
@@ -67,7 +64,7 @@ class TestSocket(BaseZMQTestCase):
                 with connect_context:
                     a.send(msg)
                     rcvd = self.recv(b)
-                    self.assertEqual(rcvd, msg)
+                    assert rcvd == msg
                 # b should now be disconnected, so sending and receiving don't work
                 with pytest.raises(zmq.Again):
                     a.send(msg, flags=zmq.DONTWAIT)
@@ -84,7 +81,7 @@ class TestSocket(BaseZMQTestCase):
                     b.connect(url)
                     a.send(msg)
                     rcvd = self.recv(b)
-                    self.assertEqual(rcvd, msg)
+                    assert rcvd == msg
                     b.disconnect(url)
                 b.connect(url)
                 # Since a is unbound from url, b is not connected to anything
@@ -105,10 +102,10 @@ class TestSocket(BaseZMQTestCase):
     def test_dir(self):
         ctx = self.Context()
         s = ctx.socket(zmq.PUB)
-        self.assertTrue('send' in dir(s))
-        self.assertTrue('IDENTITY' in dir(s))
-        self.assertTrue('AFFINITY' in dir(s))
-        self.assertTrue('FD' in dir(s))
+        assert 'send' in dir(s)
+        assert 'IDENTITY' in dir(s)
+        assert 'AFFINITY' in dir(s)
+        assert 'FD' in dir(s)
         s.close()
         ctx.term()
 
@@ -120,42 +117,55 @@ class TestSocket(BaseZMQTestCase):
 
     def test_bind_unicode(self):
         s = self.socket(zmq.PUB)
-        p = s.bind_to_random_port(unicode("tcp://*"))
+        p = s.bind_to_random_port("tcp://*")
 
     def test_connect_unicode(self):
         s = self.socket(zmq.PUB)
-        s.connect(unicode("tcp://127.0.0.1:5555"))
+        s.connect("tcp://127.0.0.1:5555")
 
     def test_bind_to_random_port(self):
         # Check that bind_to_random_port do not hide useful exception
         ctx = self.Context()
-        c = ctx.socket(zmq.PUB)
+        s = ctx.socket(zmq.PUB)
         # Invalid format
         try:
-            c.bind_to_random_port('tcp:*')
+            s.bind_to_random_port('tcp:*')
         except zmq.ZMQError as e:
-            self.assertEqual(e.errno, zmq.EINVAL)
+            assert e.errno == zmq.EINVAL
         # Invalid protocol
         try:
-            c.bind_to_random_port('rand://*')
+            s.bind_to_random_port('rand://*')
         except zmq.ZMQError as e:
-            self.assertEqual(e.errno, zmq.EPROTONOSUPPORT)
+            assert e.errno == zmq.EPROTONOSUPPORT
+
+        s.close()
+        ctx.term()
+
+    def test_bind_connect_addr_error(self):
+        with self.socket(zmq.PUSH) as s:
+            url = "tcp://1.2.3.4.5:1234567"
+            with pytest.raises(zmq.ZMQError) as exc:
+                s.bind(url)
+            assert url in str(exc.value)
+
+            url = "noproc://no/such/file"
+            with pytest.raises(zmq.ZMQError) as exc:
+                s.connect(url)
+            assert url in str(exc.value)
 
     def test_identity(self):
         s = self.context.socket(zmq.PULL)
         self.sockets.append(s)
         ident = b'identity\0\0'
         s.identity = ident
-        self.assertEqual(s.get(zmq.IDENTITY), ident)
+        assert s.get(zmq.IDENTITY) == ident
 
     def test_unicode_sockopts(self):
         """test setting/getting sockopts with unicode strings"""
         topic = "tést"
-        if str is not unicode:
-            topic = topic.decode('utf8')
         p, s = self.create_bound_pair(zmq.PUB, zmq.SUB)
-        self.assertEqual(s.send_unicode, s.send_unicode)
-        self.assertEqual(p.recv_unicode, p.recv_unicode)
+        assert s.send_unicode == s.send_unicode
+        assert p.recv_unicode == p.recv_unicode
         self.assertRaises(TypeError, s.setsockopt, zmq.SUBSCRIBE, topic)
         self.assertRaises(TypeError, s.setsockopt, zmq.IDENTITY, topic)
         s.setsockopt_unicode(zmq.IDENTITY, topic, 'utf16')
@@ -167,12 +177,12 @@ class TestSocket(BaseZMQTestCase):
         identb = s.getsockopt(zmq.IDENTITY)
         identu = identb.decode('utf16')
         identu2 = s.getsockopt_unicode(zmq.IDENTITY, 'utf16')
-        self.assertEqual(identu, identu2)
+        assert identu == identu2
         time.sleep(0.1)  # wait for connection/subscription
         p.send_unicode(topic, zmq.SNDMORE)
         p.send_unicode(topic * 2, encoding='latin-1')
-        self.assertEqual(topic, s.recv_unicode())
-        self.assertEqual(topic * 2, s.recv_unicode(encoding='latin-1'))
+        assert topic == s.recv_unicode()
+        assert topic * 2 == s.recv_unicode(encoding='latin-1')
 
     def test_int_sockopts(self):
         "test integer sockopts"
@@ -183,19 +193,19 @@ class TestSocket(BaseZMQTestCase):
             default_hwm = 1000
         p, s = self.create_bound_pair(zmq.PUB, zmq.SUB)
         p.setsockopt(zmq.LINGER, 0)
-        self.assertEqual(p.getsockopt(zmq.LINGER), 0)
+        assert p.getsockopt(zmq.LINGER) == 0
         p.setsockopt(zmq.LINGER, -1)
-        self.assertEqual(p.getsockopt(zmq.LINGER), -1)
-        self.assertEqual(p.hwm, default_hwm)
+        assert p.getsockopt(zmq.LINGER) == -1
+        assert p.hwm == default_hwm
         p.hwm = 11
-        self.assertEqual(p.hwm, 11)
+        assert p.hwm == 11
         # p.setsockopt(zmq.EVENTS, zmq.POLLIN)
-        self.assertEqual(p.getsockopt(zmq.EVENTS), zmq.POLLOUT)
-        self.assertRaisesErrno(zmq.EINVAL, p.setsockopt, zmq.EVENTS, 2 ** 7 - 1)
-        self.assertEqual(p.getsockopt(zmq.TYPE), p.socket_type)
-        self.assertEqual(p.getsockopt(zmq.TYPE), zmq.PUB)
-        self.assertEqual(s.getsockopt(zmq.TYPE), s.socket_type)
-        self.assertEqual(s.getsockopt(zmq.TYPE), zmq.SUB)
+        assert p.getsockopt(zmq.EVENTS) == zmq.POLLOUT
+        self.assertRaisesErrno(zmq.EINVAL, p.setsockopt, zmq.EVENTS, 2**7 - 1)
+        assert p.getsockopt(zmq.TYPE) == p.socket_type
+        assert p.getsockopt(zmq.TYPE) == zmq.PUB
+        assert s.getsockopt(zmq.TYPE) == s.socket_type
+        assert s.getsockopt(zmq.TYPE) == zmq.SUB
 
         # check for overflow / wrong type:
         errors = []
@@ -205,10 +215,15 @@ class TestSocket(BaseZMQTestCase):
             value = getattr(constants, name)
             if isinstance(value, int):
                 backref[value] = name
-        for opt in zmq.constants.int_sockopts.union(zmq.constants.int64_sockopts):
-            sopt = backref[opt]
-            if sopt.startswith(
+        for opt in zmq.constants.SocketOption:
+            if opt._opt_type not in {
+                zmq.constants._OptType.int,
+                zmq.constants._OptType.int64,
+            }:
+                continue
+            if opt.name.startswith(
                 (
+                    'HWM',
                     'ROUTER',
                     'XPUB',
                     'TCP',
@@ -223,6 +238,13 @@ class TestSocket(BaseZMQTestCase):
                     'VMCI_BUFFER_MIN_SIZE',
                     'VMCI_BUFFER_MAX_SIZE',
                     'VMCI_CONNECT_TIMEOUT',
+                    'BLOCKY',
+                    'IN_BATCH_SIZE',
+                    'OUT_BATCH_SIZE',
+                    'WSS_TRUST_SYSTEM',
+                    'ONLY_FIRST_SUBSCRIBE',
+                    'PRIORITY',
+                    'RECONNECT_STOP',
                 )
             ):
                 # some sockopts are write-only
@@ -230,12 +252,12 @@ class TestSocket(BaseZMQTestCase):
             try:
                 n = p.getsockopt(opt)
             except zmq.ZMQError as e:
-                errors.append("getsockopt(zmq.%s) raised '%s'." % (sopt, e))
+                errors.append(f"getsockopt({opt!r}) raised {e}.")
             else:
-                if n > 2 ** 31:
+                if n > 2**31:
                     errors.append(
-                        "getsockopt(zmq.%s) returned a ridiculous value."
-                        " It is probably the wrong type." % sopt
+                        f"getsockopt({opt!r}) returned a ridiculous value."
+                        " It is probably the wrong type."
                     )
         if errors:
             self.fail('\n'.join([''] + errors))
@@ -258,24 +280,22 @@ class TestSocket(BaseZMQTestCase):
         p = self.context.socket(zmq.PUB)
         self.sockets.append(p)
         p.setsockopt(zmq.LINGER, 11)
-        self.assertEqual(p.getsockopt(zmq.LINGER), 11)
+        assert p.getsockopt(zmq.LINGER) == 11
 
     def test_send_unicode(self):
         "test sending unicode objects"
         a, b = self.create_bound_pair(zmq.PAIR, zmq.PAIR)
         self.sockets.extend([a, b])
         u = "çπ§"
-        if str is not unicode:
-            u = u.decode('utf8')
         self.assertRaises(TypeError, a.send, u, copy=False)
         self.assertRaises(TypeError, a.send, u, copy=True)
         a.send_unicode(u)
         s = b.recv()
-        self.assertEqual(s, u.encode('utf8'))
-        self.assertEqual(s.decode('utf8'), u)
+        assert s == u.encode('utf8')
+        assert s.decode('utf8') == u
         a.send_unicode(u, encoding='utf16')
         s = b.recv_unicode(encoding='utf16')
-        self.assertEqual(s, u)
+        assert s == u
 
     def test_send_multipart_check_type(self):
         "check type on all frames in send_multipart"
@@ -284,7 +304,7 @@ class TestSocket(BaseZMQTestCase):
         self.assertRaises(TypeError, a.send_multipart, [b'a', 5])
         a.send_multipart([b'b'])
         rcvd = self.recv_multipart(b)
-        self.assertEqual(rcvd, [b'b'])
+        assert rcvd == [b'b']
 
     @skip_pypy
     def test_tracker(self):
@@ -322,38 +342,38 @@ class TestSocket(BaseZMQTestCase):
             if p1.done:
                 break
             time.sleep(0.1)
-        self.assertEqual(p1.done, True)
-        self.assertEqual(msg, [b'something'])
+        assert p1.done == True
+        assert msg == [b'something']
         msg = self.recv_multipart(b)
         for i in range(10):
             if p2.done:
                 break
             time.sleep(0.1)
-        self.assertEqual(p2.done, True)
-        self.assertEqual(msg, [b'something', b'else'])
+        assert p2.done == True
+        assert msg == [b'something', b'else']
         m = zmq.Frame(b"again", copy=False, track=True)
-        self.assertEqual(m.tracker.done, False)
+        assert m.tracker.done == False
         p1 = a.send(m, copy=False)
         p2 = a.send(m, copy=False)
-        self.assertEqual(m.tracker.done, False)
-        self.assertEqual(p1.done, False)
-        self.assertEqual(p2.done, False)
+        assert m.tracker.done == False
+        assert p1.done == False
+        assert p2.done == False
         msg = self.recv_multipart(b)
-        self.assertEqual(m.tracker.done, False)
-        self.assertEqual(msg, [b'again'])
+        assert m.tracker.done == False
+        assert msg == [b'again']
         msg = self.recv_multipart(b)
-        self.assertEqual(m.tracker.done, False)
-        self.assertEqual(msg, [b'again'])
-        self.assertEqual(p1.done, False)
-        self.assertEqual(p2.done, False)
-        pm = m.tracker
+        assert m.tracker.done == False
+        assert msg == [b'again']
+        assert p1.done == False
+        assert p2.done == False
+        m.tracker
         del m
         for i in range(10):
             if p1.done:
                 break
             time.sleep(0.1)
-        self.assertEqual(p1.done, True)
-        self.assertEqual(p2.done, True)
+        assert p1.done == True
+        assert p2.done == True
         m = zmq.Frame(b'something', track=False)
         self.assertRaises(ValueError, a.send, m, copy=False, track=True)
 
@@ -366,7 +386,7 @@ class TestSocket(BaseZMQTestCase):
         self.assertRaisesErrno(zmq.ENOTSOCK, s.setsockopt, zmq.SUBSCRIBE, b'')
         self.assertRaisesErrno(zmq.ENOTSOCK, s.send, b'asdf')
         self.assertRaisesErrno(zmq.ENOTSOCK, s.recv)
-        del ctx
+        ctx.term()
 
     def test_attr(self):
         """set setting/getting sockopts as attributes"""
@@ -374,9 +394,9 @@ class TestSocket(BaseZMQTestCase):
         self.sockets.append(s)
         linger = 10
         s.linger = linger
-        self.assertEqual(linger, s.linger)
-        self.assertEqual(linger, s.getsockopt(zmq.LINGER))
-        self.assertEqual(s.fd, s.getsockopt(zmq.FD))
+        assert linger == s.linger
+        assert linger == s.getsockopt(zmq.LINGER)
+        assert s.fd == s.getsockopt(zmq.FD)
 
     def test_bad_attr(self):
         s = self.context.socket(zmq.DEALER)
@@ -402,15 +422,15 @@ class TestSocket(BaseZMQTestCase):
 
             def __init__(self, *a, **kw):
                 self.a = -1
-                super(S, self).__init__(*a, **kw)
+                super().__init__(*a, **kw)
 
         s = S(self.context, zmq.REP)
         self.sockets.append(s)
-        self.assertEqual(s.a, -1)
+        assert s.a == -1
         s.a = 1
-        self.assertEqual(s.a, 1)
+        assert s.a == 1
         a = s.a
-        self.assertEqual(a, 1)
+        assert a == 1
 
     def test_recv_multipart(self):
         a, b = self.create_bound_pair()
@@ -419,7 +439,7 @@ class TestSocket(BaseZMQTestCase):
             a.send(msg)
         time.sleep(0.1)
         for i in range(3):
-            self.assertEqual(self.recv_multipart(b), [msg])
+            assert self.recv_multipart(b) == [msg]
 
     def test_close_after_destroy(self):
         """s.close() after ctx.destroy() should be fine"""
@@ -429,23 +449,23 @@ class TestSocket(BaseZMQTestCase):
         # reaper is not instantaneous
         time.sleep(1e-2)
         s.close()
-        self.assertTrue(s.closed)
+        assert s.closed
 
     def test_poll(self):
         a, b = self.create_bound_pair()
-        tic = time.time()
+        time.time()
         evt = a.poll(POLL_TIMEOUT)
-        self.assertEqual(evt, 0)
+        assert evt == 0
         evt = a.poll(POLL_TIMEOUT, zmq.POLLOUT)
-        self.assertEqual(evt, zmq.POLLOUT)
+        assert evt == zmq.POLLOUT
         msg = b'hi'
         a.send(msg)
         evt = b.poll(POLL_TIMEOUT)
-        self.assertEqual(evt, zmq.POLLIN)
+        assert evt == zmq.POLLIN
         msg2 = self.recv(b)
         evt = b.poll(POLL_TIMEOUT)
-        self.assertEqual(evt, 0)
-        self.assertEqual(msg2, msg)
+        assert evt == 0
+        assert msg2 == msg
 
     def test_ipc_path_max_length(self):
         """IPC_PATH_MAX_LEN is a sensible value"""
@@ -453,8 +473,8 @@ class TestSocket(BaseZMQTestCase):
             raise SkipTest("IPC_PATH_MAX_LEN undefined")
 
         msg = "Surprising value for IPC_PATH_MAX_LEN: %s" % zmq.IPC_PATH_MAX_LEN
-        self.assertTrue(zmq.IPC_PATH_MAX_LEN > 30, msg)
-        self.assertTrue(zmq.IPC_PATH_MAX_LEN < 1025, msg)
+        assert zmq.IPC_PATH_MAX_LEN > 30, msg
+        assert zmq.IPC_PATH_MAX_LEN < 1025, msg
 
     def test_ipc_path_max_length_msg(self):
         if zmq.IPC_PATH_MAX_LEN == 0:
@@ -463,9 +483,9 @@ class TestSocket(BaseZMQTestCase):
         s = self.context.socket(zmq.PUB)
         self.sockets.append(s)
         try:
-            s.bind('ipc://{0}'.format('a' * (zmq.IPC_PATH_MAX_LEN + 1)))
+            s.bind('ipc://{}'.format('a' * (zmq.IPC_PATH_MAX_LEN + 1)))
         except zmq.ZMQError as e:
-            self.assertTrue(str(zmq.IPC_PATH_MAX_LEN) in e.strerror)
+            assert str(zmq.IPC_PATH_MAX_LEN) in e.strerror
 
     @mark.skipif(windows, reason="ipc not supported on Windows.")
     def test_ipc_path_no_such_file_or_directory_message(self):
@@ -474,7 +494,7 @@ class TestSocket(BaseZMQTestCase):
         self.sockets.append(s)
         invalid_path = '/foo/bar'
         with pytest.raises(zmq.ZMQError) as error:
-            s.bind('ipc://{0}'.format(invalid_path))
+            s.bind(f'ipc://{invalid_path}')
         assert error.value.errno == errno.ENOENT
         error_message = str(error.value)
         assert invalid_path in error_message
@@ -485,14 +505,14 @@ class TestSocket(BaseZMQTestCase):
         for stype in (zmq.PUB, zmq.ROUTER, zmq.SUB, zmq.REQ, zmq.DEALER):
             s = self.context.socket(stype)
             s.hwm = 100
-            self.assertEqual(s.hwm, 100)
+            assert s.hwm == 100
             if zmq3:
                 try:
-                    self.assertEqual(s.sndhwm, 100)
+                    assert s.sndhwm == 100
                 except AttributeError:
                     pass
                 try:
-                    self.assertEqual(s.rcvhwm, 100)
+                    assert s.rcvhwm == 100
                 except AttributeError:
                     pass
             s.close()
@@ -501,10 +521,10 @@ class TestSocket(BaseZMQTestCase):
         s = self.socket(zmq.PUB)
         scopy = copy.copy(s)
         sdcopy = copy.deepcopy(s)
-        self.assertTrue(scopy._shadow)
-        self.assertTrue(sdcopy._shadow)
-        self.assertEqual(s.underlying, scopy.underlying)
-        self.assertEqual(s.underlying, sdcopy.underlying)
+        assert scopy._shadow
+        assert sdcopy._shadow
+        assert s.underlying == scopy.underlying
+        assert s.underlying == sdcopy.underlying
         s.close()
 
     def test_send_buffer(self):
@@ -520,16 +540,20 @@ class TestSocket(BaseZMQTestCase):
         p = self.socket(zmq.PUSH)
         p.bind("tcp://127.0.0.1:5555")
         p2 = zmq.Socket.shadow(p.underlying)
-        self.assertEqual(p.underlying, p2.underlying)
+        assert p.underlying == p2.underlying
         s = self.socket(zmq.PULL)
-        s2 = zmq.Socket.shadow(s.underlying)
-        self.assertNotEqual(s.underlying, p.underlying)
-        self.assertEqual(s.underlying, s2.underlying)
+        s2 = zmq.Socket.shadow(s)
+        assert s2._shadow_obj is s
+        assert s.underlying != p.underlying
+        assert s2.underlying == s.underlying
+        s3 = zmq.Socket(s)
+        assert s3._shadow_obj is s
+        assert s3.underlying == s.underlying
         s2.connect("tcp://127.0.0.1:5555")
         sent = b'hi'
         p2.send(sent)
         rcvd = self.recv(s2)
-        self.assertEqual(rcvd, sent)
+        assert rcvd == sent
 
     def test_shadow_pyczmq(self):
         try:
@@ -546,7 +570,7 @@ class TestSocket(BaseZMQTestCase):
         b.connect("inproc://a")
         a.send(b'hi')
         rcvd = self.recv(b)
-        self.assertEqual(rcvd, b'hi')
+        assert rcvd == b'hi'
 
     def test_subscribe_method(self):
         pub, sub = self.create_bound_pair(zmq.PUB, zmq.SUB)
@@ -563,21 +587,21 @@ class TestSocket(BaseZMQTestCase):
         self.recv(sub)
         pub.send(b'prefixmessage')
         msg = self.recv(sub)
-        self.assertEqual(msg, b'prefixmessage')
+        assert msg == b'prefixmessage'
         sub.unsubscribe('prefix')
         pub.send(b'prefixmessage')
         events = p.poll(1000)
-        self.assertEqual(events, [])
+        assert events == []
 
     # CI often can't handle how much memory PyPy uses on this test
     @mark.skipif(
-        (pypy and on_ci) or (sys.maxsize < 2 ** 32) or (windows),
+        (pypy and on_ci) or (sys.maxsize < 2**32) or (windows),
         reason="only run on 64b and not on CI.",
     )
     @mark.large
     def test_large_send(self):
         c = os.urandom(1)
-        N = 2 ** 31 + 1
+        N = 2**31 + 1
         try:
             buf = c * N
         except MemoryError as e:
@@ -653,8 +677,8 @@ if have_gevent and not windows:
             with warnings.catch_warnings(record=True) as w:
                 s.rcvtimeo = 5
             s.close()
-            self.assertEqual(len(w), 1)
-            self.assertEqual(w[0].category, UserWarning)
+            assert len(w) == 1
+            assert w[0].category == UserWarning
 
         @mark.skipif(not hasattr(zmq, 'SNDTIMEO'), reason="requires SNDTIMEO")
         def test_warn_get_timeo(self):
@@ -662,5 +686,5 @@ if have_gevent and not windows:
             with warnings.catch_warnings(record=True) as w:
                 s.sndtimeo
             s.close()
-            self.assertEqual(len(w), 1)
-            self.assertEqual(w[0].category, UserWarning)
+            assert len(w) == 1
+            assert w[0].category == UserWarning
