@@ -3,6 +3,7 @@
 
 
 from unittest import TestCase
+
 import zmq
 from zmq.sugar import version
 
@@ -11,32 +12,32 @@ class TestVersion(TestCase):
     def test_pyzmq_version(self):
         vs = zmq.pyzmq_version()
         vs2 = zmq.__version__
-        self.assertTrue(isinstance(vs, str))
+        assert isinstance(vs, str)
         if zmq.__revision__:
-            self.assertEqual(vs, '@'.join(vs2, zmq.__revision__))
+            assert vs == '@'.join(vs2, zmq.__revision__)
         else:
-            self.assertEqual(vs, vs2)
+            assert vs == vs2
         if version.VERSION_EXTRA:
-            self.assertTrue(version.VERSION_EXTRA in vs)
-            self.assertTrue(version.VERSION_EXTRA in vs2)
+            assert version.VERSION_EXTRA in vs
+            assert version.VERSION_EXTRA in vs2
 
     def test_pyzmq_version_info(self):
         info = zmq.pyzmq_version_info()
-        self.assertTrue(isinstance(info, tuple))
+        assert isinstance(info, tuple)
         for n in info[:3]:
-            self.assertTrue(isinstance(n, int))
+            assert isinstance(n, int)
         if version.VERSION_EXTRA:
-            self.assertEqual(len(info), 4)
-            self.assertEqual(info[-1], float('inf'))
+            assert len(info) == 4
+            assert info[-1] == float('inf')
         else:
-            self.assertEqual(len(info), 3)
+            assert len(info) == 3
 
     def test_zmq_version_info(self):
         info = zmq.zmq_version_info()
-        self.assertTrue(isinstance(info, tuple))
+        assert isinstance(info, tuple)
         for n in info[:3]:
-            self.assertTrue(isinstance(n, int))
+            assert isinstance(n, int)
 
     def test_zmq_version(self):
         v = zmq.zmq_version()
-        self.assertTrue(isinstance(v, str))
+        assert isinstance(v, str)
