@@ -8,7 +8,7 @@ Authors
 # Copyright (C) PyZMQ Developers
 # Distributed under the terms of the Modified BSD License.
 
-import sys
+import warnings
 
 bytes = bytes
 unicode = str
@@ -17,9 +17,14 @@ basestring = (str,)
 
 def cast_bytes(s, encoding='utf8', errors='strict'):
     """cast unicode or bytes to bytes"""
+    warnings.warn(
+        "zmq.utils.strtypes is deprecated in pyzmq 23.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     if isinstance(s, bytes):
         return s
-    elif isinstance(s, unicode):
+    elif isinstance(s, str):
         return s.encode(encoding, errors)
     else:
         raise TypeError("Expected unicode or bytes, got %r" % s)
@@ -27,15 +32,20 @@ def cast_bytes(s, encoding='utf8', errors='strict'):
 
 def cast_unicode(s, encoding='utf8', errors='strict'):
     """cast bytes or unicode to unicode"""
+    warnings.warn(
+        "zmq.utils.strtypes is deprecated in pyzmq 23.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     if isinstance(s, bytes):
         return s.decode(encoding, errors)
-    elif isinstance(s, unicode):
+    elif isinstance(s, str):
         return s
     else:
         raise TypeError("Expected unicode or bytes, got %r" % s)
 
 
-# give short 'b' alias for cast_bytes, so that we can use fake b('stuff')
+# give short 'b' alias for cast_bytes, so that we can use fake b'stuff'
 # to simulate b'stuff'
 b = asbytes = cast_bytes
 u = cast_unicode
